/*-------------------------------------------------------------------------------------------------\
| Misc CAN Node Firmware
|
| File:					MSC_INI.C		Initialization Code
|
| Global Routines:	Initialize()
|
| Local Routines:		None
|
\-------------------------------------------------------------------------------------------------*/

#include	<pic.h>
#include "msc.h"

//-------------------------------------------------------------------------------------------------
//  Define Locally Global Variables (variables accessible by functions in this file only)
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------

	// None

/*-------------------------------------------------------------------------------------------------\
| Function:   Initialize
|
| Purpose:    Initialize the ports, peripherals and variables
|
| Discussion:	This function initializes the ports and peripherals of the PIC16F877
|             
| Expects:    To be called after any type of reset.
|             Interrupts to be disabled.
|					
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Stolen from ADG code.
\--------------------------------------------------------------------------------------------------*/

void Initialize (void)
{

		char	i, j, k;

	// First we need to bring all of the PORTs into their correct state. 

	// Note that we set the PORT registers FIRST, and then the TRIS registers. In this order,
	// we can be sure that the output pins come "up" out of tri-state in the right state.

	ADCON1 = 0b00000110;		// Turn off PORTA ADC inputs (necessary for TRISA to do anything)
			//   ||||||||
			//   |||||||+----------- PCFG0 = 0110 (No ADC - all digital)
			//   ||||||+------------ PCFG1
			//   |||||+------------- PCFG2         
			//   ||||+-------------- PCFG3         
			//   |||+--------------- <Not implemented on PIC>
			//   ||+---------------- <Not implemented on PIC>
			//   |+----------------- <Not implemented on PIC>
			//   +------------------ ADFM
	PORTA = 0;

	TRISA = 0;					// PORTA all outputs.

	RBPU = 0;					// Enable PortB pullups (for inputs only)
	PORTB = 0;
	TRISB =   0b00001001;	//PORTB all outputs except RB0 which is the INT interrupt intput
				// ||||||||
				// |||||||+----------- RB0: In  <- INT input from CAN chip
				// ||||||+------------ RB1: Out -> 
				// |||||+------------- RB2: Out -> 
				// ||||+-------------- RB3: In  -> Button (active Low) 
				// |||+--------------- RB4: Out -> 
				// ||+---------------- RB5: Out -> 
				// |+----------------- RB6: MPLAB ICD port
	       	// +------------------ RB7: MPLAB ICD port
	INTEDG = 1;					// RB0/INT interrupt triggers on rising edge
	
	PORTC = 0b00000011;
			 // ||||||||
			 // |||||||+----------- /RD = 1 (off)
		    // ||||||+------------ /WR = 1 (off)
		    // |||||+------------- -
		    // ||||+-------------- -
		    // |||+--------------- -
			 // ||+---------------- -
		    // |+----------------- -
	       // +------------------ -
	TRISC = 0;
			 // ||||||||
			 // |||||||+----------- Out -> /RD
		    // ||||||+------------ Out -> /WR
		    // |||||+------------- Out -> 
		    // ||||+-------------- Out -> 
		    // |||+--------------- Out -> 
			 // ||+---------------- Out -> 
		    // |+----------------- Out -> 
	       // +------------------ Out -> 


	PORTD = 0;					// PORTD is all outputs (data bus)
	TRISD = 0;

	PORTE = 0b00000010;
			 // ||||||||
			 // |||||||+----------- RE0 = /RSTCAN = 0 (reset)
		    // ||||||+------------ RE1 = /CSCAN = 1 (deselected)
		    // |||||+------------- RE2 = ALE = 0 (off)
		    // ||||+-------------- <Not implemented on PIC>
		    // |||+--------------- <Not implemented on PIC>
			 // ||+---------------- <Not implemented on PIC>
		    // |+----------------- <Not implemented on PIC>
	       // +------------------ <Not implemented on PIC>
	TRISE = 0;					// PORTE all outputs and parallel port peripheral is off
	
	// ---------------------------------------------------------------------------

	// First, let things settle (~100ms) for a moment.
		
	for (i = 0; i < 15; ++i)
		for (j = 0; j < 255; ++j)
			for (k = 0; k < 255; ++k)
				asm("nop");

	// ---------------------------------------------------------------------------
	
	gbPowerUp = 1;
	gbTxReady = 1; 	// The transmit buffer is obviously empty after a reset
	
	#ifdef STATUSLEDS
		PORTA = 0xFF;	// TURN OFF status lights after power up.
	#endif

	// ---------------------------------------------------------------------------
	// Initialize Timer2 to make a 1KHz (1ms) interrupt given a 20MHz Fosc.
	
	T2CON = 0b00100101;
			 // ||||||||
			 // |||||||+----------- T2CKPS0 = 01  (Prescaler is to 1:4)
		    // ||||||+------------ T2CKPS1
		    // |||||+------------- TMR2ON  = 1   (Timer2 is on)
		    // ||||+-------------- TOUTPS0 = 0100 (1:5 Postscaler)
		    // |||+--------------- TOUTPS1
			 // ||+---------------- TOUTPS2
		    // |+----------------- TOUTPS3
	       // +------------------ Unimplemented
	PR2 = 249;		// Set the period register for Timer to 250 (1us * 4 * 250 = 1ms => 1kHz)

	// ---------------------------------------------------------------------------

	// Clear any pending interrupts
	
	TMR2IF = 0;				// Clear the Timer2 interrupt
	INTF = 0;				// Clear the RB0/INT interrupt
	
	// Turn on all of the interrupt enables and finally the global enables.

	TMR2IE = 1;				// Enable interrupt on Timer2 to PR2 match
	PEIE = 1;				// Enable peripheral interrupts (for Timer2)
	INTE = 1;				// Enable the RB0/INT interrupt
	GIE = 1;					// Enable global interrupts

}


