/*-------------------------------------------------------------------------------------------------\
| Misc CAN Node Firmware
|
| File:					MSC_MSC.C		Misc. Functions
|
| Global Routines:	LoadTxQueue_RA(struct message)
| 
| Local Routines:		None
|							
\-------------------------------------------------------------------------------------------------*/

#include	<pic.h>
#include "msc.h"

//-------------------------------------------------------------------------------------------------
//  Define Locally Global Variables (variables accessible by functions in this file only)
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------

	// None


/*-------------------------------------------------------------------------------------------------\
| Function:		LoadTxQueue
|
| Purpose:		Loads a CAN message into the transmit queue (gmTransmitQueue).
|
| Discussion:	The gmTransmitQueue[] (which is TXQUEUELENGTH messages long) is a ring FIFO queue that
|					contains messages that are to be sent out to the SJA. The routine which actually
|					transfers the message from gmTransmitQueue[] to the SJA 1000 CAN chip is SendMessage()
|					which is called from main.
|             
| Expects:		To be called by any routine wanting to send a CAN message.
|					
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/07/2002 | Stolen from ADG code.
\--------------------------------------------------------------------------------------------------*/

void LoadTxQueue (struct message mMessage)
{
	// Catch any array index overflow problem

	if (gcTxQueueIn >= TXQUEUELENGTH)
	{
		gcSystemError = 10;
		gcTxQueueIn = TXQUEUELENGTH - 1;
		#ifdef STATUSLEDS
			ERROR_LED = LED_ON;
		#endif
	}
	
	// Save the message in the queue

	gmTransmitQueue[gcTxQueueIn].cFrameInfo = mMessage.cFrameInfo;
	gmTransmitQueue[gcTxQueueIn].cMsgID1    = mMessage.cMsgID1;
	gmTransmitQueue[gcTxQueueIn].cMsgID2    = mMessage.cMsgID2;
	gmTransmitQueue[gcTxQueueIn].cData[0]   = mMessage.cData[0];
	gmTransmitQueue[gcTxQueueIn].cData[1]   = mMessage.cData[1];
	gmTransmitQueue[gcTxQueueIn].cData[2]   = mMessage.cData[2];
	gmTransmitQueue[gcTxQueueIn].cData[3]   = mMessage.cData[3];		
	gmTransmitQueue[gcTxQueueIn].cData[4]   = mMessage.cData[4];
	gmTransmitQueue[gcTxQueueIn].cData[5]   = mMessage.cData[5];
	gmTransmitQueue[gcTxQueueIn].cData[6]   = mMessage.cData[6];
	gmTransmitQueue[gcTxQueueIn].cData[7]   = mMessage.cData[7];
	
	// Now increment the index and see how bad that screws us up
	
	++gcTxQueueIn;
	if (gcTxQueueIn >= TXQUEUELENGTH)
		gcTxQueueIn = 0;

	if (gcTxQueueIn == gcTxQueueOut)
	{
		// The index in equals the index out so we have a ring buffer overflow condition.
		// Back up the pointer and signal the error condition.
		
		if (gcTxQueueIn == 0)
			gcTxQueueIn = (TXQUEUELENGTH - 1);
		else
			--gcTxQueueIn;

		gcSystemError = 8;
		#ifdef STATUSLEDS
			ERROR_LED = LED_ON;
		#endif
	}

	// Note that main calls SendMessage() when gcTxQueueIn != gcTxQueueOut which is always
	// the case when leaving this routine.
}

