/*-------------------------------------------------------------------------------------------------\
| Misc CAN Node Firmware
|
| File:					MSC_SJA.C		Misc. Functions
|
| Global Routines:	Init_SJA ()
| 
| Local Routines:		None
|							
\-------------------------------------------------------------------------------------------------*/

#include	<pic.h>
#include "msc.h"
#include "sja.h"

//-------------------------------------------------------------------------------------------------
//  Define Locally Global Variables (variables accessible by functions in this file only)
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local #defines
//-------------------------------------------------------------------------------------------------

	// None

//-------------------------------------------------------------------------------------------------	
//  Local Function Prototypes
//-------------------------------------------------------------------------------------------------

	void		Write_SJA		(char cAdrs, char cData);
	char		Read_SJA			(char cAdrs);



/*-------------------------------------------------------------------------------------------------\
| Function:		SendCount
|
| Purpose:		Sends the number of times this message has been sent.
|
| Discussion:	
|             
| Expects:		
|
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/07/2002 | Created.
\--------------------------------------------------------------------------------------------------*/
		
void SendCount (void)
{
	static struct message mMessage;
	static char	cDigit1;
	static char	cDigit2;
	static char	cDigit3;
	static char	cDigit4;
	static char	cDigit5;
	static char	cDigit6;
	static char	cDigit7;
	static char	cDigit8;

	// Count in BCD with 8 digits
	
	++cDigit1;
	if (cDigit1 > 9)
	{
		cDigit1 = 0;
		++cDigit2;
		if (cDigit2 > 9)
		{
			cDigit2 = 0;
			++cDigit3;
			if (cDigit3 > 9)
			{
				cDigit3 = 0;
				++cDigit4;
				if (cDigit4 > 9)
				{
					cDigit4 = 0;
					++cDigit5;
					if (cDigit5 > 9)
					{
						cDigit5 = 0;
						++cDigit6;
						if (cDigit6 > 9)
						{
							cDigit6 = 0;
							++cDigit7;
							if (cDigit7 > 9)
							{
								cDigit7 = 0;
								++cDigit8;
								if (cDigit8 > 9)
									cDigit8 = 0;
							}
						}
					}
				}
			}
		}
	}

	// Message ID  = 0x7FC = 0b111 1111 1100
	
	mMessage.cFrameInfo = 0b00001000;
	mMessage.cMsgID1 =    0b11111111;
	mMessage.cMsgID2 =    0b10000000;

	// Send a bunch of random things as data
	
	mMessage.cData[0] = cDigit8;
	mMessage.cData[1] = cDigit7;
	mMessage.cData[2] = cDigit6;
	mMessage.cData[3] = cDigit5;
	mMessage.cData[4] = cDigit4;
	mMessage.cData[5] = cDigit3;
	mMessage.cData[6] = cDigit2;
	mMessage.cData[7] = cDigit1;

	// Load it into the transmit queue
	
	LoadTxQueue(mMessage);
}


/*-------------------------------------------------------------------------------------------------\
| Function:		ButtonPushed
|
| Purpose:		Sends a message (once) when RB3 goes low.
|
| Discussion:	Send a bunch of junk in the message packet
|             
| Expects:		
|
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/07/2002 | Created.
\--------------------------------------------------------------------------------------------------*/
		
void ButtonPushed (void)
{
	static struct message mMessage;
	static bit	bPushed;
	
	if (RB3 == 0)
	{
		if (!bPushed)
		{
			// Message ID: 0x7FD = 0b111 1111 1101
	
			mMessage.cFrameInfo = 0b00000100;
			mMessage.cMsgID1 =    0b11111111;
			mMessage.cMsgID2 =    0b10100000;

			// Send a bunch of random things as data
			
			mMessage.cData[0] = gcSystemError;
			mMessage.cData[1] = Read_SJA(SJA_ECC);
			mMessage.cData[2] = Read_SJA(SJA_RXERR);
			mMessage.cData[3] = Read_SJA(SJA_TXERR);

			// Load it into the transmit queue
	
			LoadTxQueue(mMessage);

			// Flag that we've been here once
			
			bPushed = 1;
		}
	}
	else
		bPushed = 0;
}


/*-------------------------------------------------------------------------------------------------\
| Function:		ServiceSJA
|
| Purpose:		Read in a message from the SJA1000 and write it to the receive queue.
|
| Discussion:	
|             
| Expects:		
|
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/07/2002 | Created.
\--------------------------------------------------------------------------------------------------*/

void ServiceSJA (void)
{
	static char	cIntReg;

	// Read the interrupt flags register
	
	cIntReg = Read_SJA(SJA_IR);

	// Data Overrun interrupt (IR.3): Set if the the SJA1000's 64byte FIFO has overflowed (the PIC wasn't
	// reading messages fast enough).
	
	if (cIntReg & 0b00001000)
	{
		#ifdef	STATUSLEDS
			OVERFLOW_LED = LED_ON;
		#endif
	}
	
	// Error Passive INterrupt (IR.5): Set if there was a normal to passive, or passive to active, error
	// transition in the SJA1000. Errors like these usually occur because a there is nothing else active
	// on the bus.
	
	if (cIntReg & 0b00100000)
	{
		#ifdef	STATUSLEDS
			CANERR_LED = LED_ON;
		#endif
	}

	// Bus Error Interrupt (IR.7): Set if there was a "hard" bus error, like a bit stuffing violation,
	// corrupted CRC, etc.
		
	if (cIntReg & 0b10000000)
	{
		gcErrArray[gcErrIndex] = Read_SJA(SJA_ECC);
		if (gcErrIndex < 8)
			++gcErrIndex;
			
		#ifdef	STATUSLEDS
			BUSERR_LED = LED_ON;
		#endif
	}

	// Transmit Interrupt (IR.1): Set if the Transmit Buffer is Empty
	
	if (cIntReg & 0b00000010)
	{
		gbTxReady = 1;
	}
	
	// Receive Interupt (IR.0): The Receive Bugger has received a new message.
	
	if (cIntReg & 0b00000001)
	{
		gbRxReady = 1;
	}

	// We've processed the interrupt; wait for the next one
	
	gbSJAInterrupt = 0;
}



/*-------------------------------------------------------------------------------------------------\
| Function:		GetMessage
|
| Purpose:		Read in a message from the SJA1000 and write it to the receive queue.
|
| Discussion:	
|             
| Expects:		
|
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/07/2002 | Created.
\--------------------------------------------------------------------------------------------------*/

void GetMessage (void)
{
	static char i;
	
	// Get the Frame information
	
	gmMessage.cFrameInfo = Read_SJA( SJA_TXF );

	// Get the Message ID
	
	gmMessage.cMsgID1 = Read_SJA( SJA_TXID1 );
	gmMessage.cMsgID2 = Read_SJA( SJA_TXID2 );

	// Clear the data array.
	
	for (i = 0; i < 9; ++i)
		gmMessage.cData[i] = 0;

	// Load in the number of data bytes unless the Remote Frame (RTR) bit is set
		
	if ( !(gmMessage.cFrameInfo & 0b01000000) )
		for (i = 0; i < (gmMessage.cFrameInfo & 0b00001111); ++i)
			gmMessage.cData[i] = Read_SJA( SJA_TXD1 + i);

	// Tell the PIC and the SJA1000 we've read the message

	gbRxReady = 0;
	
	Write_SJA (SJA_CMR, 0b00000100 );
							 // ||||||||
							 // |||||||+----------- TR = 0				-> No message to transmit
							 // ||||||+------------ AT = 0				-> Do not abort transmit
							 // |||||+------------- RRB = 1				-> Release this message from the receive buffer
							 // ||||+-------------- CDO = 0				-> Do not clear a data overrun error
							 // |||+--------------- SRR = 0				-> Do not enable "self reception"
							 // ||+---------------- <reserved>
							 // |+----------------- <reserved>
							 // +------------------ <reserved>
		
	// Check if there are more messages to read

	// #### DON'T NEED TO DO THIS? DOES THE SJA REFLAG THE INTERRUPT IF THERE'S ANOTHER MESSAGE
	// IN THE FIFO? TEST THIS OUT. ######
}




	
/*-------------------------------------------------------------------------------------------------\
| Function:		SendMessage
|
| Purpose:		Send a CAN message via the SJA1000
|
| Discussion:	Sends a message from the TransmitQueue to the SJA1000
|             
| Expects:		
|
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/06/2002 | Created.
\--------------------------------------------------------------------------------------------------*/

void SendMessage (void)
{
	static char i;

	// First, make sure the transmit buffer has been released.
	
	if ( (Read_SJA(SJA_SR) & 0b00000100) )
	{
		// Write the Frame type & # data bytes
	
		Write_SJA(SJA_TXF, gmTransmitQueue[gcTxQueueOut].cFrameInfo);
		
		// Write the Message ID
	
		Write_SJA(SJA_TXID1, gmTransmitQueue[gcTxQueueOut].cMsgID1);
		Write_SJA(SJA_TXID2, gmTransmitQueue[gcTxQueueOut].cMsgID2);

		// Write the data bytes out

		for (i = 0; i < 8; ++i)
			Write_SJA((SJA_TXD1 + i), gmTransmitQueue[gcTxQueueOut].cData[i]);

		// Done with the data transfer; Now tell the SJA to send it out

		Write_SJA (SJA_CMR, 0b00000001 );
								 // ||||||||
								 // |||||||+----------- TR = 1				-> Transmit message
								 // ||||||+------------ AT = 0				-> Do not abort transmit
								 // |||||+------------- RRB = 0				-> Do not release the receive buffer
								 // ||||+-------------- CDO = 0				-> Do not clear a data overrun error
								 // |||+--------------- SRR = 0				-> Do not enable "self reception"
								 // ||+---------------- <reserved>
								 // |+----------------- <reserved>
								 // +------------------ <reserved>

		// Done sending the message; increment the out index
	
		++gcTxQueueOut;
		if (gcTxQueueOut >= TXQUEUELENGTH)
			gcTxQueueOut = 0;
	}
}


/*-------------------------------------------------------------------------------------------------\
| Function:		Read_SJA_Regs
|
| Purpose:		Read the SJA1000's registers into an array
|
| Discussion:	
|             
| Expects:		
|
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/06/2002 | Created.
\--------------------------------------------------------------------------------------------------*/

void Read_SJA_Regs (void)
{
	char i;

	for (i = 0; i <= 31; ++i)
		cSJARegisters[i] = Read_SJA(i);
}




/*-------------------------------------------------------------------------------------------------\
| Function:		Init_SJA
|
| Purpose:		Initializes the SJA1000
|
| Discussion:	Initialize the SJA1000 into "PeliCAN" mode, 1mbps,
|             
| Expects:		
|
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Created.
\--------------------------------------------------------------------------------------------------*/

void Init_SJA (void)
{
	char i, j, k;
	
	RSTCAN_PIN = 1;
	
	// Let the SJA1000 come online (~100ms)
		
	for (i = 0; i < 2; ++i)
		for (j = 0; j < 255; ++j)
			for (k = 0; k < 255; ++k)
				asm("nop");

	// The SJA1000 is now in "Reset Mode" (as opposed to "Operational Mode") and may now
	// be configured properly. Note that there should be no reason to configure the CMR
	// or MOD register since the reset default values are correct.

	Write_SJA(SJA_CDR, 0b11000111 );
							// ||||||||
							// |||||||+----------- CD.0 = 111 			-> Clock Out = (Fosc/1)
							// ||||||+------------ CD.1
							// |||||+------------- CD.2
							// ||||+-------------- "clock off" = 0 	-> Clock out enabled
							// |||+--------------- <reserved>
							// ||+---------------- RXINTEND = 0 		-> TX1 not used as interrupt out
							// |+----------------- CBP = 1 				-> Bypass input comparator & use RX0 only
							// +------------------ "CAN Mode" = 1		-> PeliCAN mode

	Write_SJA (SJA_OCR, 0b00011010 );
							 // ||||||||
							 // |||||||+----------- OCMODE0 = 10 		-> Normal Output (not clock, not test, not biphase)
							 // ||||||+------------ OCMODE1
							 // |||||+------------- OCPOL0 = 110 		-> Set TX0 = TXD
							 // ||||+-------------- OCTN0
							 // |||+--------------- OCTP0
							 // ||+---------------- OCPOL1 = 000 		-> Set TX1 = OFF (Note: 111 -> TX1 = !TXD)
							 // |+----------------- OCTN1
							 // +------------------ OCTP1

	// Bus Settings for 125kbps
	
//	Write_SJA (SJA_BTR0, 0b01000111 );
							  // ||||||||
							  // |||||||+----------- BRP.0 = 000111	-> baud Rate Prescaler: Tq = 2*Tclk*(7+1) = 800ns
							  // ||||||+------------ BRP.1
							  // |||||+------------- BRP.2
							  // ||||+-------------- BRP.3
							  // |||+--------------- BRP.4
							  // ||+---------------- BRP.5 
							  // |+----------------- SJW.0 = 01       -> Synchronization Jump Width = 1+1 -> 2Tq = 1.6us 
							  // +------------------ SJW.1					(800ns = 10% of Tbit = 1/125kbps = 8us)

//	Write_SJA (SJA_BTR1, 0b10100101 );
							  // ||||||||
							  // |||||||+----------- TSEG1.0 = 0101	-> Time Segment 1 = (5+1)Tq = 4.8us 
							  // ||||||+------------ TSEG1.1
							  // |||||+------------- TSEG1.2
							  // ||||+-------------- TSEG1.3
							  // |||+--------------- TSEG2.0 = 010		-> Time Segment 2 = (2+1)Tq = 2.4us
							  // ||+---------------- TSEG2.1
							  // |+----------------- TSEG2.2
							  // +------------------ SAM = 1				-> Multiple samples per Tbit (good for low speed)

	// Bus Settings for 1Mbps 
	
	Write_SJA (SJA_BTR0, 0b00000000 );
							  // ||||||||
							  // |||||||+----------- BRP.0 = 000000	-> baud Rate Prescaler: Tq = 2*Tclk*(0+1) = 100ns
							  // ||||||+------------ BRP.1
							  // |||||+------------- BRP.2
							  // ||||+-------------- BRP.3
							  // |||+--------------- BRP.4
							  // ||+---------------- BRP.5 
							  // |+----------------- SJW.0 = 00       -> Synchronization Jump Width = 0+1 -> 1Tq = 100ns 
							  // +------------------ SJW.1					(100ns = 10% of Tbit = 1/1Mbps = 1us)

	Write_SJA (SJA_BTR1, 0b00100101 );
							  // ||||||||
							  // |||||||+----------- TSEG1.0 = 0101	-> Time Segment 1 = (5+1)Tq = 600ns
							  // ||||||+------------ TSEG1.1
							  // |||||+------------- TSEG1.2
							  // ||||+-------------- TSEG1.3
							  // |||+--------------- TSEG2.0 = 010		-> Time Segment 2 = (2+1)Tq = 300ns
							  // ||+---------------- TSEG2.1
							  // |+----------------- TSEG2.2
							  // +------------------ SAM = 0				-> One sample per Tbit (good for high speed)

	Write_SJA (SJA_IER, 0b10101001 );
							 // ||||||||
							 // |||||||+----------- RIE = 1				-> Receive interrupts enabled
							 // ||||||+------------ TIE = 0				-> Transmit interrupts disabled
							 // |||||+------------- EIE = 0				-> Error warning interrupts disabled
							 // ||||+-------------- DOIE = 1				-> Data overrun interrupts enabled
							 // |||+--------------- WUIE = 0				-> Wake-up interrupts disabled 
							 // ||+---------------- EPIE = 1				-> Error passive Interrupts enabled
							 // |+----------------- ALIE = 0				-> Arbitration lost interrupts disabled
							 // +------------------ BEIE = 1				-> Bus error interrupts disabled

	// The chip should be configured. Now enter the two Acceptance Filters and Masks

	// Acceptance Filter #1: 0x7FF

	Write_SJA (SJA_ACR1, 0b11100000 );
	Write_SJA (SJA_AMR1, 0b00011111 );
							  // ||||||||
							  // |||||||+----------- DB1.4 = CAN Message Data Byte 1, bit 4
							  // ||||||+------------ DB1.5 = CAN Message Data Byte 1, bit 5
							  // |||||+------------- DB1.6 = CAN Message Data Byte 1, bit 6
							  // ||||+-------------- DB1.7 = CAN Message Data Byte 1, bit 7
							  // |||+--------------- RTR   = CAN Message RTR bit
							  // ||+---------------- ID.18 = CAN Message ID bit 01
							  // |+----------------- ID.19 = CAN Message ID bit 02
							  // +------------------ ID.20 = CAN Message ID bit 03
	
	Write_SJA (SJA_ACR0, 0b11111111 );
	Write_SJA (SJA_AMR0, 0b00000000 );
							  // ||||||||
							  // |||||||+----------- ID.21 = CAN Message ID bit 04
							  // ||||||+------------ ID.22 = CAN Message ID bit 05
							  // |||||+------------- ID.23 = CAN Message ID bit 06
							  // ||||+-------------- ID.24 = CAN Message ID bit 07
							  // |||+--------------- ID.25 = CAN Message ID bit 08
							  // ||+---------------- ID.26 = CAN Message ID bit 09
							  // |+----------------- ID.27 = CAN Message ID bit 10
							  // +------------------ ID.28 = CAN Message ID bit 11

	// Acceptance Filter #2: 0x7FE
	
	Write_SJA (SJA_ACR3, 0b11000000 );
	Write_SJA (SJA_AMR3, 0b00011111 );
							  // ||||||||
							  // |||||||+----------- DB1.4 = CAN Message Data Byte 1, bit 0
							  // ||||||+------------ DB1.5 = CAN Message Data Byte 1, bit 1
							  // |||||+------------- DB1.6 = CAN Message Data Byte 1, bit 2
							  // ||||+-------------- DB1.7 = CAN Message Data Byte 1, bit 3
							  // |||+--------------- RTR   = CAN Message RTR bit
							  // ||+---------------- ID.18 = CAN Message ID bit 01
							  // |+----------------- ID.19 = CAN Message ID bit 02
							  // +------------------ ID.20 = CAN Message ID bit 03
	
	Write_SJA (SJA_ACR2, 0b11111111 );
	Write_SJA (SJA_AMR2, 0b00000000 );
							  // ||||||||
							  // |||||||+----------- ID.21 = CAN Message ID bit 04
							  // ||||||+------------ ID.22 = CAN Message ID bit 05
							  // |||||+------------- ID.23 = CAN Message ID bit 06
							  // ||||+-------------- ID.24 = CAN Message ID bit 07
							  // |||+--------------- ID.25 = CAN Message ID bit 08
							  // ||+---------------- ID.26 = CAN Message ID bit 09
							  // |+----------------- ID.27 = CAN Message ID bit 10
							  // +------------------ ID.28 = CAN Message ID bit 11

	// Now turn on the SJA1000 by switching to "Operational Mode"
	
	Write_SJA (SJA_MOD, 0b00000000 );
							 // ||||||||
							 // |||||||+----------- RM = 0				-> Switch to "Operational Mode"
							 // ||||||+------------ LOM = 0				-> Listen Only Mode off
							 // |||||+------------- STM = 0				-> Self Test Mode off
							 // ||||+-------------- AFM = 0				-> Dual Acceptance Filter mode
							 // |||+--------------- SM = 0				-> Sleep Mode off
							 // ||+---------------- <reserved>
							 // |+----------------- <reserved>
							 // +------------------ <reserved>
}


/*-------------------------------------------------------------------------------------------------\
| Function:		Write_SJA
|
| Purpose:		Writes a byte to the SJA1000
|
| Discussion:	Requires an address and data byte
|             
| Expects:		TRISD = 0
|					
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Created.
\--------------------------------------------------------------------------------------------------*/

void Write_SJA (char cAdrs, char cData)
{
	// Disable interrupts and select the SJA1000

	GIE = 0;
	CSCAN_PIN = 0;
	
	// Latch the address into the SJA1000
	
	PORTD = cAdrs;
	ALE_PIN = 1;
	ALE_PIN = 0;

	// Write the data to the SJA1000
	
	PORTD = cData;
	WR_PIN = 0;
	WR_PIN = 1;

	// Deselect the SJA1000 and re-enable interrupts
	
	CSCAN_PIN = 1;
	GIE = 1;
}


/*-------------------------------------------------------------------------------------------------\
| Function:		Read_SJA
|
| Purpose:		Read a byte to the SJA1000
|
| Discussion:	Requires an address and data byte. Turns off interrupts since the interrupts may
|					also read from the SJA1000.
|             
| Expects:		TRISD = 0
|					NOT TO BE CALLED FROM AN INTERRUPT ROUTINE
|					
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Created.
\--------------------------------------------------------------------------------------------------*/

char	Read_SJA	(char cAdrs)
{

	char	cData;

	// Turn off interrupts and select the SJA1000

	GIE = 0;
	CSCAN_PIN = 0;
	
	// Latch the address into the SJA1000
	
	PORTD = cAdrs;
	ALE_PIN = 1;
	ALE_PIN = 0;

	// Read the data to the SJA1000

	TRISD = 0xFF;
	RD_PIN = 0;
	cData = PORTD;
	RD_PIN = 1;
	TRISD = 0;
	
	// Deselect the SJA1000 and re-enable interrupts
	
	CSCAN_PIN = 1;
	GIE = 1;

	// Return the data
	
	return	cData;
}



/*-------------------------------------------------------------------------------------------------\
| Function:		Read_SJA_Int
|
| Purpose:		Read a byte to the SJA1000
|
| Discussion:	Requires an address and data byte
|             
| Expects:		TRISD = 0
|					ONLY TO BE CALLED FROM AN INTERRUPT ROUTINE (and thus interrupts disabled)
|					
|   Date     | Notes
|------------|---------------------------------------------------------------------------------------
| 01/05/2002 | Created.
\--------------------------------------------------------------------------------------------------*/

char	Read_SJA_Int	(char cAdrs)
{

	char	cData;

	// Select the SJA1000
	
	CSCAN_PIN = 0;
	
	// Latch the address into the SJA1000
	
	PORTD = cAdrs;
	ALE_PIN = 1;
	ALE_PIN = 0;

	// Read the data to the SJA1000

	TRISD = 0xFF;
	RD_PIN = 0;
	cData = PORTD;
	RD_PIN = 1;
	TRISD = 0;
	
	// Deselect the SJA1000
	
	CSCAN_PIN = 1;

	// Return the data
	
	return	cData;
}

	 
