/*-------------------------------------------------------------------------------------------------\
| Misc CAN Node Firmware
|
| File:					SJA.H			Header File for the SJA1000 routines
|
| Global Routines:	None
|
| Local Routines:		None
\-------------------------------------------------------------------------------------------------*/


//-------------------------------------------------------------------------------------------------
//  Enums
//-------------------------------------------------------------------------------------------------

enum	sja_registers	// Read/Write
{
	SJA_MOD = 0,		// Mode
	SJA_CMR,				// Command
	SJA_SR,				// Status		
	SJA_IR,				// Interrupt
	SJA_IER,				// Interrupt Enable
	
	SJA_BTR0 = 6,		// Bus Timing 0
	SJA_BTR1,			// Bus Timing 1
	SJA_OCR,				// Output Control
	
	SJA_ALC = 11,		// Arbitration Lost Capture
	SJA_ECC,				// Error Code Capture
	SJA_EWLR,			// Error Warning Limit
	SJA_RXERR,			// RX Error Counter
	SJA_TXERR,			// TX Error Counter

	SJA_RMC = 29,		// RX Message Counter
	SJA_RBSA,			// RX Buffer Start Address
	SJA_CDR,				// Clock Divider Register

	SJA_ACR0	= 16,		// Acceptance Code Register 0
	SJA_ACR1,			// Acceptance Code Register 1
	SJA_ACR2, 			// Acceptance Code Register 2
	SJA_ACR3,			// Acceptance Code Register 3
	SJA_AMR0,			// Acceptance Mask Register 0
	SJA_AMR1,			// Acceptance Mask Register 1
	SJA_AMR2,			// Acceptance Mask Register 2
	SJA_AMR3,			// Acceptance Mask Register 3

	SJA_RXF = 16,		// RX Frame Information
	SJA_RXID1,			// RX Identifier 1
	SJA_RXID2,			// RX Identifier 2
	SJA_RXD1,			// RX Data 1
	SJA_RXD2,			// RX Data 2
	SJA_RXD3,			// RX Data 3
	SJA_RXD4,			// RX Data 4
	SJA_RXD5,			// RX Data 5
	SJA_RXD6,			// RX Data 6
	SJA_RXD7,			// RX Data 7
	SJA_RXD8,			// RX Data 8

	SJA_TXF = 16,		// TX Frame Information
	SJA_TXID1,			// TX Identifier 1
	SJA_TXID2,			// TX Identifier 2
	SJA_TXD1,			// TX Data 1
	SJA_TXD2,			// TX Data 2
	SJA_TXD3,			// TX Data 3
	SJA_TXD4,			// TX Data 4
	SJA_TXD5,			// TX Data 5
	SJA_TXD6,			// TX Data 6
	SJA_TXD7,			// TX Data 7
	SJA_TXD8,			// TX Data 8
};


//-------------------------------------------------------------------------------------------------
// #defines
//-------------------------------------------------------------------------------------------------

#define	RD_PIN			RC0		// /RD = read  (active low)
#define 	WR_PIN			RC1		// /WR = write (active low)
#define	RSTCAN_PIN		RE0		// /RESET = Reset line (active low)
#define	CSCAN_PIN		RE1		// /CS = chip select (active low)
#define	ALE_PIN			RE2		// ALE = Adress Latch Enable

